@@ -45,8 +45,6 @@
 #endif
 
 #include <gst/gl/gstglapi.h>
-
-#include "gstglelements.h"
 #include "gstglfiltercube.h"
 #include "gstglutils.h"
 
@@ -70,8 +68,6 @@ enum
 #define gst_gl_filter_cube_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstGLFilterCube, gst_gl_filter_cube,
     GST_TYPE_GL_FILTER, DEBUG_INIT);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (glfiltercube, "glfiltercube",
-    GST_RANK_NONE, GST_TYPE_GL_FILTER_CUBE, gl_element_init (plugin));
 
 static void gst_gl_filter_cube_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -179,7 +175,7 @@ gst_gl_filter_cube_class_init (GstGLFilterCubeClass * klass)
           "Specifies the distance from the viewer to the far clipping plane",
           0.0, 1000.0, 100.0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_set_static_metadata (element_class, "OpenGL cube filter",
+  gst_element_class_set_metadata (element_class, "OpenGL cube filter",
       "Filter/Effect/Video", "Map input texture on the 6 cube faces",
       "Julien Isorce <julien.isorce@gmail.com>");
 
@@ -315,10 +311,6 @@ gst_gl_filter_cube_gl_start (GstGLBaseFilter * filter)
   gchar *frag_str;
   gboolean ret;
 
-  cube_filter->xrot = 0.0;
-  cube_filter->yrot = 0.0;
-  cube_filter->zrot = 0.0;
-
   frag_str =
       g_strdup_printf ("%s%s",
       gst_gl_shader_string_get_highest_precision (context,
@@ -441,6 +433,10 @@ _callback (gpointer stuff)
   GstGLFilterCube *cube_filter = GST_GL_FILTER_CUBE (filter);
   GstGLFuncs *gl = GST_GL_BASE_FILTER (filter)->context->gl_vtable;
 
+  static GLfloat xrot = 0;
+  static GLfloat yrot = 0;
+  static GLfloat zrot = 0;
+
   const GLfloat matrix[] = {
     0.5f, 0.0f, 0.0f, 0.0f,
     0.0f, 0.5f, 0.0f, 0.0f,
@@ -458,12 +454,9 @@ _callback (gpointer stuff)
   gl->ActiveTexture (GL_TEXTURE0);
   gl->BindTexture (GL_TEXTURE_2D, cube_filter->in_tex->tex_id);
   gst_gl_shader_set_uniform_1i (cube_filter->shader, "s_texture", 0);
-  gst_gl_shader_set_uniform_1f (cube_filter->shader, "xrot_degree",
-      cube_filter->xrot);
-  gst_gl_shader_set_uniform_1f (cube_filter->shader, "yrot_degree",
-      cube_filter->yrot);
-  gst_gl_shader_set_uniform_1f (cube_filter->shader, "zrot_degree",
-      cube_filter->zrot);
+  gst_gl_shader_set_uniform_1f (cube_filter->shader, "xrot_degree", xrot);
+  gst_gl_shader_set_uniform_1f (cube_filter->shader, "yrot_degree", yrot);
+  gst_gl_shader_set_uniform_1f (cube_filter->shader, "zrot_degree", zrot);
   gst_gl_shader_set_uniform_matrix_4fv (cube_filter->shader, "u_matrix", 1,
       GL_FALSE, matrix);
 
@@ -505,9 +498,9 @@ _callback (gpointer stuff)
 
   gl->Disable (GL_DEPTH_TEST);
 
-  cube_filter->xrot += 0.3f;
-  cube_filter->yrot += 0.2f;
-  cube_filter->zrot += 0.4f;
+  xrot += 0.3f;
+  yrot += 0.2f;
+  zrot += 0.4f;
 
   return TRUE;
 }
