@@ -29,7 +29,7 @@
 G_BEGIN_DECLS
 
 /* buffer pool functions */
-GST_EXPORT
+GST_GL_API
 GType gst_gl_buffer_pool_get_type (void);
 #define GST_TYPE_GL_BUFFER_POOL      (gst_gl_buffer_pool_get_type())
 #define GST_IS_GL_BUFFER_POOL(obj)   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_GL_BUFFER_POOL))
@@ -66,12 +66,12 @@ struct _GstGLBufferPoolClass
   gpointer _padding[GST_PADDING];
 };
 
-GST_EXPORT
+GST_GL_API
 GstBufferPool *gst_gl_buffer_pool_new (GstGLContext * context);
 
-GST_EXPORT
+GST_GL_API
 GstGLAllocationParams * gst_buffer_pool_config_get_gl_allocation_params    (GstStructure * config);
-GST_EXPORT
+GST_GL_API
 void                    gst_buffer_pool_config_set_gl_allocation_params    (GstStructure * config,
                                                                             GstGLAllocationParams * params);
 
