@@ -55,26 +55,22 @@ static void gst_video_aggregator_reset_qos (GstVideoAggregator * vagg);
 
 #define DEFAULT_PAD_ZORDER 0
 #define DEFAULT_PAD_REPEAT_AFTER_EOS FALSE
-#define DEFAULT_PAD_MAX_LAST_BUFFER_REPEAT GST_CLOCK_TIME_NONE
 enum
 {
   PROP_PAD_0,
   PROP_PAD_ZORDER,
   PROP_PAD_REPEAT_AFTER_EOS,
-  PROP_PAD_MAX_LAST_BUFFER_REPEAT,
 };
 
 
 struct _GstVideoAggregatorPadPrivate
 {
   GstBuffer *buffer;
-  GstCaps *caps;
   GstVideoFrame prepared_frame;
 
   /* properties */
   guint zorder;
   gboolean repeat_after_eos;
-  GstClockTime max_last_buffer_repeat;
 
   /* Subclasses can force an alpha channel in the (input thus output)
    * colorspace format */
@@ -84,7 +80,6 @@ struct _GstVideoAggregatorPadPrivate
   GstClockTime end_time;
 
   GstVideoInfo pending_vinfo;
-  GstCaps *pending_caps;
 };
 
 
@@ -104,9 +99,6 @@ gst_video_aggregator_pad_get_property (GObject * object, guint prop_id,
     case PROP_PAD_REPEAT_AFTER_EOS:
       g_value_set_boolean (value, pad->priv->repeat_after_eos);
       break;
-    case PROP_PAD_MAX_LAST_BUFFER_REPEAT:
-      g_value_set_uint64 (value, pad->priv->max_last_buffer_repeat);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -125,34 +117,26 @@ gst_video_aggregator_pad_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstVideoAggregatorPad *pad = GST_VIDEO_AGGREGATOR_PAD (object);
+  GstVideoAggregator *vagg =
+      GST_VIDEO_AGGREGATOR (gst_pad_get_parent (GST_PAD (pad)));
 
   switch (prop_id) {
-    case PROP_PAD_ZORDER:{
-      GstVideoAggregator *vagg =
-          GST_VIDEO_AGGREGATOR (gst_pad_get_parent (GST_PAD (pad)));
-      if (vagg) {
-        GST_OBJECT_LOCK (vagg);
-        pad->priv->zorder = g_value_get_uint (value);
-        GST_ELEMENT (vagg)->sinkpads =
-            g_list_sort (GST_ELEMENT (vagg)->sinkpads,
-            (GCompareFunc) pad_zorder_compare);
-        GST_OBJECT_UNLOCK (vagg);
-        gst_object_unref (vagg);
-      } else {
-        pad->priv->zorder = g_value_get_uint (value);
-      }
+    case PROP_PAD_ZORDER:
+      GST_OBJECT_LOCK (vagg);
+      pad->priv->zorder = g_value_get_uint (value);
+      GST_ELEMENT (vagg)->sinkpads = g_list_sort (GST_ELEMENT (vagg)->sinkpads,
+          (GCompareFunc) pad_zorder_compare);
+      GST_OBJECT_UNLOCK (vagg);
       break;
-    }
     case PROP_PAD_REPEAT_AFTER_EOS:
       pad->priv->repeat_after_eos = g_value_get_boolean (value);
       break;
-    case PROP_PAD_MAX_LAST_BUFFER_REPEAT:
-      pad->priv->max_last_buffer_repeat = g_value_get_uint64 (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
+
+  gst_object_unref (vagg);
 }
 
 static GstFlowReturn
@@ -163,7 +147,6 @@ _flush_pad (GstAggregatorPad * aggpad, GstAggregator * aggregator)
 
   gst_video_aggregator_reset_qos (vagg);
   gst_buffer_replace (&pad->priv->buffer, NULL);
-  gst_caps_replace (&pad->priv->caps, NULL);
   pad->priv->start_time = -1;
   pad->priv->end_time = -1;
 
@@ -216,21 +199,6 @@ gst_video_aggregator_pad_clean_frame (GstVideoAggregatorPad * pad,
   }
 }
 
-static GstSample *
-gst_video_aggregator_peek_next_sample (GstAggregator * agg,
-    GstAggregatorPad * aggpad)
-{
-  GstVideoAggregatorPad *vaggpad = GST_VIDEO_AGGREGATOR_PAD (aggpad);
-  GstSample *res = NULL;
-
-  if (vaggpad->priv->buffer) {
-    res = gst_sample_new (vaggpad->priv->buffer, vaggpad->priv->caps,
-        &aggpad->segment, NULL);
-  }
-
-  return res;
-}
-
 static void
 gst_video_aggregator_pad_class_init (GstVideoAggregatorPadClass * klass)
 {
@@ -250,35 +218,6 @@ gst_video_aggregator_pad_class_init (GstVideoAggregatorPadClass * klass)
           DEFAULT_PAD_REPEAT_AFTER_EOS,
           G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstVideoAggregatorPad::max-last-buffer-repeat:
-   *
-   * Repeat last buffer for time (in ns, -1 = until EOS).
-   * The default behaviour is for the last buffer received on a pad to be
-   * aggregated until a new buffer is received.
-   *
-   * Setting this property causes the last buffer to be discarded once the
-   * running time of the output buffer is `max-last-buffer-repeat` nanoseconds
-   * past its end running time. When the buffer didn't have a duration, the
-   * comparison is made against its running start time.
-   *
-   * This is useful in live scenarios: when a stream encounters a temporary
-   * networking problem, a #GstVideoAggregator subclass can then fall back to
-   * displaying a lower z-order stream, or the background.
-   *
-   * Setting this property doesn't affect the behaviour on EOS.
-   *
-   * Since: 1.18
-   */
-  g_object_class_install_property (gobject_class,
-      PROP_PAD_MAX_LAST_BUFFER_REPEAT,
-      g_param_spec_uint64 ("max-last-buffer-repeat", "Max Last Buffer Repeat",
-          "Repeat last buffer for time (in ns, -1=until EOS), "
-          "behaviour on EOS is not affected", 0, G_MAXUINT64,
-          DEFAULT_PAD_MAX_LAST_BUFFER_REPEAT,
-          G_PARAM_READWRITE | GST_PARAM_MUTABLE_PLAYING |
-          G_PARAM_STATIC_STRINGS));
-
   aggpadclass->flush = GST_DEBUG_FUNCPTR (_flush_pad);
   aggpadclass->skip_buffer =
       GST_DEBUG_FUNCPTR (gst_video_aggregator_pad_skip_buffer);
@@ -294,7 +233,6 @@ gst_video_aggregator_pad_init (GstVideoAggregatorPad * vaggpad)
 
   vaggpad->priv->zorder = DEFAULT_PAD_ZORDER;
   vaggpad->priv->repeat_after_eos = DEFAULT_PAD_REPEAT_AFTER_EOS;
-  vaggpad->priv->max_last_buffer_repeat = DEFAULT_PAD_MAX_LAST_BUFFER_REPEAT;
   memset (&vaggpad->priv->prepared_frame, 0, sizeof (GstVideoFrame));
 }
 
@@ -305,8 +243,8 @@ gst_video_aggregator_pad_init (GstVideoAggregatorPad * vaggpad)
  * Checks if the pad currently has a buffer queued that is going to be used
  * for the current output frame.
  *
- * This must only be called from the #GstVideoAggregatorClass::aggregate_frames virtual method,
- * or from the #GstVideoAggregatorPadClass::prepare_frame virtual method of the aggregator pads.
+ * This must only be called from the aggregate_frames() virtual method,
+ * or from the prepare_frame() virtual method of the aggregator pads.
  *
  * Returns: %TRUE if the pad has currently a buffer queued
  */
@@ -325,10 +263,10 @@ gst_video_aggregator_pad_has_current_buffer (GstVideoAggregatorPad * pad)
  * Returns the currently queued buffer that is going to be used
  * for the current output frame.
  *
- * This must only be called from the #GstVideoAggregatorClass::aggregate_frames virtual method,
- * or from the #GstVideoAggregatorPadClass::prepare_frame virtual method of the aggregator pads.
+ * This must only be called from the aggregate_frames() virtual method,
+ * or from the prepare_frame() virtual method of the aggregator pads.
  *
- * The return value is only valid until #GstVideoAggregatorClass::aggregate_frames or #GstVideoAggregatorPadClass::prepare_frame
+ * The return value is only valid until aggregate_frames() or prepare_frames()
  * returns.
  *
  * Returns: (transfer none): The currently queued buffer
@@ -348,10 +286,10 @@ gst_video_aggregator_pad_get_current_buffer (GstVideoAggregatorPad * pad)
  * Returns the currently prepared video frame that has to be aggregated into
  * the current output frame.
  *
- * This must only be called from the #GstVideoAggregatorClass::aggregate_frames virtual method,
- * or from the #GstVideoAggregatorPadClass::prepare_frame virtual method of the aggregator pads.
+ * This must only be called from the aggregate_frames() virtual method,
+ * or from the prepare_frame() virtual method of the aggregator pads.
  *
- * The return value is only valid until #GstVideoAggregatorClass::aggregate_frames or #GstVideoAggregatorPadClass::prepare_frame
+ * The return value is only valid until aggregate_frames() or prepare_frames()
  * returns.
  *
  * Returns: (transfer none): The currently prepared video frame
@@ -401,9 +339,6 @@ enum
 
 struct _GstVideoAggregatorConvertPadPrivate
 {
-  /* The following fields are only used from the aggregate thread and when
-   * initializing / finalizing */
-
   /* Converter, if NULL no conversion is done */
   GstVideoConverter *convert;
 
@@ -411,8 +346,6 @@ struct _GstVideoAggregatorConvertPadPrivate
   GstVideoInfo conversion_info;
   GstBuffer *converted_buffer;
 
-  /* The following fields are accessed from the property setters / getters,
-   * and as such are protected with the object lock */
   GstStructure *converter_config;
   gboolean converter_config_changed;
 };
@@ -442,9 +375,7 @@ static void
 {
   GstVideoAggregatorConvertPad *pad = GST_VIDEO_AGGREGATOR_CONVERT_PAD (vpad);
 
-  GST_OBJECT_LOCK (pad);
   pad->priv->converter_config_changed = TRUE;
-  GST_OBJECT_UNLOCK (pad);
 }
 
 static gboolean
@@ -456,7 +387,6 @@ gst_video_aggregator_convert_pad_prepare_frame (GstVideoAggregatorPad * vpad,
   GstVideoFrame frame;
 
   /* Update/create converter as needed */
-  GST_OBJECT_LOCK (pad);
   if (pad->priv->converter_config_changed) {
     GstVideoAggregatorConvertPadClass *klass =
         GST_VIDEO_AGGREGATOR_CONVERT_PAD_GET_CLASS (pad);
@@ -468,31 +398,33 @@ gst_video_aggregator_convert_pad_prepare_frame (GstVideoAggregatorPad * vpad,
       return FALSE;
     pad->priv->converter_config_changed = FALSE;
 
-    pad->priv->conversion_info = conversion_info;
+    if (!pad->priv->conversion_info.finfo
+        || !gst_video_info_is_equal (&conversion_info,
+            &pad->priv->conversion_info)) {
+      pad->priv->conversion_info = conversion_info;
 
-    if (pad->priv->convert)
-      gst_video_converter_free (pad->priv->convert);
-    pad->priv->convert = NULL;
+      if (pad->priv->convert)
+        gst_video_converter_free (pad->priv->convert);
+      pad->priv->convert = NULL;
 
-    if (!gst_video_info_is_equal (&vpad->info, &pad->priv->conversion_info)) {
-      pad->priv->convert =
-          gst_video_converter_new (&vpad->info, &pad->priv->conversion_info,
-          pad->priv->converter_config ? gst_structure_copy (pad->priv->
-              converter_config) : NULL);
-      if (!pad->priv->convert) {
-        GST_WARNING_OBJECT (pad, "No path found for conversion");
-        return FALSE;
+      if (!gst_video_info_is_equal (&vpad->info, &pad->priv->conversion_info)) {
+        pad->priv->convert =
+            gst_video_converter_new (&vpad->info, &pad->priv->conversion_info,
+            pad->priv->converter_config ? gst_structure_copy (pad->
+                priv->converter_config) : NULL);
+        if (!pad->priv->convert) {
+          GST_WARNING_OBJECT (pad, "No path found for conversion");
+          return FALSE;
+        }
+
+        GST_DEBUG_OBJECT (pad, "This pad will be converted from %d to %d",
+            GST_VIDEO_INFO_FORMAT (&vpad->info),
+            GST_VIDEO_INFO_FORMAT (&pad->priv->conversion_info));
+      } else {
+        GST_DEBUG_OBJECT (pad, "This pad will not need conversion");
       }
-
-      GST_DEBUG_OBJECT (pad, "This pad will be converted from %s to %s",
-          gst_video_format_to_string (GST_VIDEO_INFO_FORMAT (&vpad->info)),
-          gst_video_format_to_string (GST_VIDEO_INFO_FORMAT (&pad->
-                  priv->conversion_info)));
-    } else {
-      GST_DEBUG_OBJECT (pad, "This pad will not need conversion");
     }
   }
-  GST_OBJECT_UNLOCK (pad);
 
   if (!gst_video_frame_map (&frame, &vpad->info, buffer, GST_MAP_READ)) {
     GST_WARNING_OBJECT (vagg, "Could not map input buffer");
@@ -705,9 +637,7 @@ void gst_video_aggregator_convert_pad_update_conversion_info
 {
   g_return_if_fail (GST_IS_VIDEO_AGGREGATOR_CONVERT_PAD (pad));
 
-  GST_OBJECT_LOCK (pad);
   pad->priv->converter_config_changed = TRUE;
-  GST_OBJECT_UNLOCK (pad);
 }
 
 /**************************************
@@ -751,10 +681,6 @@ struct _GstVideoAggregatorPrivate
   GstCaps *current_caps;
 
   gboolean live;
-
-  /* The (ordered) list of #GstVideoFormatInfo supported by the aggregation
-     method (from the srcpad template caps). */
-  GPtrArray *supported_formats;
 };
 
 /* Can't use the G_DEFINE_TYPE macros because we need the
@@ -796,35 +722,6 @@ gst_video_aggregator_get_instance_private (GstVideoAggregator * self)
   return (G_STRUCT_MEMBER_P (self, video_aggregator_private_offset));
 }
 
-static gboolean
-gst_video_aggregator_supports_format (GstVideoAggregator * vagg,
-    GstVideoFormat format)
-{
-  gint i;
-
-  for (i = 0; i < vagg->priv->supported_formats->len; i++) {
-    GstVideoFormatInfo *format_info = vagg->priv->supported_formats->pdata[i];
-
-    if (GST_VIDEO_FORMAT_INFO_FORMAT (format_info) == format)
-      return TRUE;
-  }
-
-  return FALSE;
-}
-
-static GstCaps *
-gst_video_aggregator_get_possible_caps_for_info (GstVideoInfo * info)
-{
-  GstStructure *s;
-  GstCaps *possible_caps = gst_video_info_to_caps (info);
-
-  s = gst_caps_get_structure (possible_caps, 0);
-  gst_structure_remove_fields (s, "width", "height", "framerate",
-      "pixel-aspect-ratio", "interlace-mode", NULL);
-
-  return possible_caps;
-}
-
 static void
 gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
     GstCaps * downstream_caps, GstVideoInfo * best_info,
@@ -834,12 +731,13 @@ gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
   GstCaps *possible_caps;
   GstVideoAggregatorPad *pad;
   gboolean need_alpha = FALSE;
-  gint best_format_number = 0, i;
+  gint best_format_number = 0;
   GHashTable *formats_table = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   GST_OBJECT_LOCK (vagg);
   for (tmp = GST_ELEMENT (vagg)->sinkpads; tmp; tmp = tmp->next) {
-    gint format_number = 0;
+    GstStructure *s;
+    gint format_number;
 
     pad = tmp->data;
 
@@ -857,29 +755,28 @@ gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
     if (GST_VIDEO_INFO_FORMAT (&pad->info) == GST_VIDEO_FORMAT_UNKNOWN)
       continue;
 
+    possible_caps = gst_video_info_to_caps (&pad->info);
+
+    s = gst_caps_get_structure (possible_caps, 0);
+    gst_structure_remove_fields (s, "width", "height", "framerate",
+        "pixel-aspect-ratio", "interlace-mode", NULL);
+
     /* Can downstream accept this format ? */
-    if (!GST_IS_VIDEO_AGGREGATOR_CONVERT_PAD (pad)) {
-      possible_caps =
-          gst_video_aggregator_get_possible_caps_for_info (&pad->info);
-      if (!gst_caps_can_intersect (downstream_caps, possible_caps)) {
-        gst_caps_unref (possible_caps);
-        continue;
-      }
+    if (!gst_caps_can_intersect (downstream_caps, possible_caps)) {
+      gst_caps_unref (possible_caps);
+      continue;
     }
 
-    /* If the format is supported, consider it very high weight */
-    if (gst_video_aggregator_supports_format (vagg,
-            GST_VIDEO_INFO_FORMAT (&pad->info))) {
-      format_number =
-          GPOINTER_TO_INT (g_hash_table_lookup (formats_table,
-              GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info))));
+    gst_caps_unref (possible_caps);
 
-      format_number += pad->info.width * pad->info.height;
+    format_number =
+        GPOINTER_TO_INT (g_hash_table_lookup (formats_table,
+            GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info))));
+    format_number += pad->info.width * pad->info.height;
 
-      g_hash_table_replace (formats_table,
-          GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info)),
-          GINT_TO_POINTER (format_number));
-    }
+    g_hash_table_replace (formats_table,
+        GINT_TO_POINTER (GST_VIDEO_INFO_FORMAT (&pad->info)),
+        GINT_TO_POINTER (format_number));
 
     /* If that pad is the first with alpha, set it as the new best format */
     if (!need_alpha && (pad->priv->needs_alpha
@@ -905,41 +802,6 @@ gst_video_aggregator_find_best_format (GstVideoAggregator * vagg,
   GST_OBJECT_UNLOCK (vagg);
 
   g_hash_table_unref (formats_table);
-
-  if (gst_video_aggregator_supports_format (vagg,
-          GST_VIDEO_INFO_FORMAT (best_info))) {
-    possible_caps = gst_video_aggregator_get_possible_caps_for_info (best_info);
-    if (gst_caps_can_intersect (downstream_caps, possible_caps)) {
-      gst_caps_unref (possible_caps);
-      return;
-    }
-    gst_caps_unref (possible_caps);
-  }
-
-  for (i = 0; i < vagg->priv->supported_formats->len; i++) {
-    GstVideoFormatInfo *format_info = vagg->priv->supported_formats->pdata[i];
-
-    if ((! !GST_VIDEO_FORMAT_INFO_HAS_ALPHA (format_info)) == (! !need_alpha)) {
-      gst_video_info_set_format (best_info, format_info->format,
-          best_info->width, best_info->height);
-      possible_caps =
-          gst_video_aggregator_get_possible_caps_for_info (best_info);
-
-      if (gst_caps_can_intersect (downstream_caps, possible_caps)) {
-        GST_INFO_OBJECT (vagg, "Using supported caps: %" GST_PTR_FORMAT,
-            possible_caps);
-        gst_caps_unref (possible_caps);
-
-        return;
-      }
-
-      gst_caps_unref (possible_caps);
-    }
-  }
-
-  GST_WARNING_OBJECT (vagg, "Nothing compatible with %" GST_PTR_FORMAT,
-      downstream_caps);
-  gst_video_info_init (best_info);
 }
 
 static GstCaps *
@@ -1035,22 +897,15 @@ gst_video_aggregator_default_update_caps (GstVideoAggregator * vagg,
   color_name = gst_video_colorimetry_to_string (&best_info.colorimetry);
 
   GST_DEBUG_OBJECT (vagg,
-      "The output format will now be : %s with chroma : %s and colorimetry %s",
-      gst_video_format_to_string (best_format),
-      GST_STR_NULL (gst_video_chroma_to_string (best_info.chroma_site)),
-      GST_STR_NULL (color_name));
+      "The output format will now be : %d with chroma : %s and colorimetry %s",
+      best_format, gst_video_chroma_to_string (best_info.chroma_site),
+      color_name);
 
   best_format_caps = gst_caps_copy (caps);
   gst_caps_set_simple (best_format_caps, "format", G_TYPE_STRING,
-      gst_video_format_to_string (best_format), NULL);
-
-  if (best_info.chroma_site != GST_VIDEO_CHROMA_SITE_UNKNOWN)
-    gst_caps_set_simple (best_format_caps, "chroma-site", G_TYPE_STRING,
-        gst_video_chroma_to_string (best_info.chroma_site), NULL);
-  if (color_name != NULL)
-    gst_caps_set_simple (best_format_caps, "colorimetry", G_TYPE_STRING,
-        color_name, NULL);
-
+      gst_video_format_to_string (best_format), "chroma-site", G_TYPE_STRING,
+      gst_video_chroma_to_string (best_info.chroma_site), "colorimetry",
+      G_TYPE_STRING, color_name, NULL);
   g_free (color_name);
   ret = gst_caps_merge (best_format_caps, gst_caps_ref (caps));
 
@@ -1063,6 +918,29 @@ gst_video_aggregator_default_update_src_caps (GstAggregator * agg,
 {
   GstVideoAggregatorClass *vagg_klass = GST_VIDEO_AGGREGATOR_GET_CLASS (agg);
   GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (agg);
+  gboolean at_least_one_pad_configured = FALSE;
+  GList *l;
+
+  GST_OBJECT_LOCK (vagg);
+  for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
+    GstVideoAggregatorPad *mpad = l->data;
+
+    if (GST_VIDEO_INFO_WIDTH (&mpad->info) == 0
+        || GST_VIDEO_INFO_HEIGHT (&mpad->info) == 0)
+      continue;
+
+    at_least_one_pad_configured = TRUE;
+  }
+  GST_OBJECT_UNLOCK (vagg);
+
+  if (!at_least_one_pad_configured) {
+    /* We couldn't decide the output video info because the sinkpads don't have
+     * all the caps yet, so we mark the pad as needing a reconfigure. This
+     * allows aggregate() to skip ahead a bit and try again later. */
+    GST_DEBUG_OBJECT (vagg, "Couldn't decide output video info");
+    gst_pad_mark_reconfigure (agg->srcpad);
+    return GST_AGGREGATOR_FLOW_NEED_DATA;
+  }
 
   g_assert (vagg_klass->update_caps);
 
@@ -1125,11 +1003,9 @@ gst_video_aggregator_default_negotiated_src_caps (GstAggregator * agg,
     gst_video_aggregator_reset_qos (vagg);
   }
 
-  GST_OBJECT_LOCK (vagg);
   vagg->info = info;
-  GST_OBJECT_UNLOCK (vagg);
 
-  finfo = info.finfo;
+  finfo = vagg->info.finfo;
 
   if (at_least_one_alpha && !(finfo->flags & GST_VIDEO_FORMAT_FLAG_ALPHA)) {
     GST_ELEMENT_ERROR (vagg, CORE, NEGOTIATION,
@@ -1150,7 +1026,7 @@ gst_video_aggregator_default_negotiated_src_caps (GstAggregator * agg,
 
     gst_aggregator_set_src_caps (agg, caps);
     latency = gst_util_uint64_scale (GST_SECOND,
-        GST_VIDEO_INFO_FPS_D (&info), GST_VIDEO_INFO_FPS_N (&info));
+        GST_VIDEO_INFO_FPS_D (&vagg->info), GST_VIDEO_INFO_FPS_N (&vagg->info));
     gst_aggregator_set_latency (agg, latency, latency);
   }
 
@@ -1231,14 +1107,12 @@ gst_video_aggregator_pad_sink_setcaps (GstPad * pad, GstObject * parent,
      * that this pad is getting configured; configure immediately to avoid
      * problems with the initial negotiation */
     vaggpad->info = info;
-    gst_caps_replace (&vaggpad->priv->caps, caps);
     gst_pad_mark_reconfigure (GST_AGGREGATOR_SRC_PAD (vagg));
   } else {
     /* this pad already had caps but received new ones; keep the new caps
      * pending until we pick the next buffer from the queue, otherwise we
      * might use an old buffer with the new caps and crash */
     vaggpad->priv->pending_vinfo = info;
-    gst_caps_replace (&vaggpad->priv->pending_caps, caps);
     GST_DEBUG_OBJECT (pad, "delaying caps change");
   }
   ret = TRUE;
@@ -1388,16 +1262,12 @@ gst_video_aggregator_pad_sink_getcaps (GstPad * pad, GstVideoAggregator * vagg,
   n = gst_caps_get_size (srccaps);
   for (i = 0; i < n; i++) {
     s = gst_caps_get_structure (srccaps, i);
-    gst_structure_set (s, "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT,
-        1, NULL);
-
-    if (GST_IS_VIDEO_AGGREGATOR_CONVERT_PAD (pad)) {
-      gst_structure_set (s, "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-          "height", GST_TYPE_INT_RANGE, 1, G_MAXINT, NULL);
-      gst_structure_remove_fields (s, "colorimetry", "chroma-site", "format",
-          "pixel-aspect-ratio", NULL);
-    }
+    gst_structure_set (s, "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+        "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+        "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
 
+    gst_structure_remove_fields (s, "colorimetry", "chroma-site", "format",
+        "pixel-aspect-ratio", NULL);
     if (has_interlace_mode)
       gst_structure_set (s, "interlace-mode", G_TYPE_STRING,
           gst_video_interlace_mode_to_string (interlace_mode), NULL);
@@ -1486,10 +1356,7 @@ gst_video_aggregator_reset (GstVideoAggregator * vagg)
   GstAggregator *agg = GST_AGGREGATOR (vagg);
   GList *l;
 
-  GST_OBJECT_LOCK (vagg);
   gst_video_info_init (&vagg->info);
-  GST_OBJECT_UNLOCK (vagg);
-
   vagg->priv->ts_offset = 0;
   vagg->priv->nframes = 0;
   vagg->priv->live = FALSE;
@@ -1503,7 +1370,6 @@ gst_video_aggregator_reset (GstVideoAggregator * vagg)
     GstVideoAggregatorPad *p = l->data;
 
     gst_buffer_replace (&p->priv->buffer, NULL);
-    gst_caps_replace (&p->priv->caps, NULL);
     p->priv->start_time = -1;
     p->priv->end_time = -1;
 
@@ -1515,12 +1381,10 @@ gst_video_aggregator_reset (GstVideoAggregator * vagg)
 static GstFlowReturn
 gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
     GstClockTime output_start_running_time,
-    GstClockTime output_end_running_time, gboolean timeout)
+    GstClockTime output_end_running_time)
 {
   GList *l;
   gboolean eos = TRUE;
-  gboolean repeat_pad_eos = FALSE;
-  gboolean has_no_repeat_pads = FALSE;
   gboolean need_more_data = FALSE;
   gboolean need_reconfigure = FALSE;
 
@@ -1543,15 +1407,10 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
 
     if (!is_eos)
       eos = FALSE;
-    if (!pad->priv->repeat_after_eos)
-      has_no_repeat_pads = TRUE;
     buf = gst_aggregator_pad_peek_buffer (bpad);
     if (buf) {
       GstClockTime start_time, end_time;
 
-    check_again:
-      GST_TRACE_OBJECT (pad, "Next buffer %" GST_PTR_FORMAT, buf);
-
       start_time = GST_BUFFER_TIMESTAMP (buf);
       if (start_time == -1) {
         gst_buffer_unref (buf);
@@ -1583,22 +1442,12 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
               "output_start_running_time.  Discarding old buffer");
           gst_buffer_replace (&pad->priv->buffer, buf);
           if (pad->priv->pending_vinfo.finfo) {
-            gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
-            gst_caps_replace (&pad->priv->pending_caps, NULL);
             pad->info = pad->priv->pending_vinfo;
             need_reconfigure = TRUE;
             pad->priv->pending_vinfo.finfo = NULL;
           }
           gst_buffer_unref (buf);
           gst_aggregator_pad_drop_buffer (bpad);
-          pad->priv->start_time = start_time;
-          if (timeout) {
-            /* If we're called for a timeout, we want to make sure we drain as
-             * much as possible any late data */
-            buf = gst_aggregator_pad_peek_buffer (bpad);
-            if (buf)
-              goto check_again;
-          }
           need_more_data = TRUE;
           continue;
         }
@@ -1606,14 +1455,11 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
         buf = gst_aggregator_pad_pop_buffer (bpad);
         gst_buffer_replace (&pad->priv->buffer, buf);
         if (pad->priv->pending_vinfo.finfo) {
-          gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
-          gst_caps_replace (&pad->priv->pending_caps, NULL);
           pad->info = pad->priv->pending_vinfo;
           need_reconfigure = TRUE;
           pad->priv->pending_vinfo.finfo = NULL;
         }
-        /* FIXME: Set end_time to something here? */
-        pad->priv->start_time = start_time;
+        /* FIXME: Set start_time and end_time to something here? */
         gst_buffer_unref (buf);
         GST_DEBUG_OBJECT (pad, "buffer duration is -1");
         continue;
@@ -1661,15 +1507,13 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
         continue;
       }
 
-      if (end_time > output_start_running_time
+      if (end_time >= output_start_running_time
           && start_time < output_end_running_time) {
         GST_DEBUG_OBJECT (pad,
             "Taking new buffer with start time %" GST_TIME_FORMAT,
             GST_TIME_ARGS (start_time));
         gst_buffer_replace (&pad->priv->buffer, buf);
         if (pad->priv->pending_vinfo.finfo) {
-          gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
-          gst_caps_replace (&pad->priv->pending_caps, NULL);
           pad->info = pad->priv->pending_vinfo;
           need_reconfigure = TRUE;
           pad->priv->pending_vinfo.finfo = NULL;
@@ -1688,8 +1532,6 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
       } else {
         gst_buffer_replace (&pad->priv->buffer, buf);
         if (pad->priv->pending_vinfo.finfo) {
-          gst_caps_replace (&pad->priv->caps, pad->priv->pending_caps);
-          gst_caps_replace (&pad->priv->pending_caps, NULL);
           pad->info = pad->priv->pending_vinfo;
           need_reconfigure = TRUE;
           pad->priv->pending_vinfo.finfo = NULL;
@@ -1708,52 +1550,25 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
       }
     } else {
       if (is_eos && pad->priv->repeat_after_eos) {
-        repeat_pad_eos = TRUE;
+        eos = FALSE;
         GST_DEBUG_OBJECT (pad, "ignoring EOS and re-using previous buffer");
         continue;
       }
 
       if (pad->priv->end_time != -1) {
         if (pad->priv->end_time <= output_start_running_time) {
+          pad->priv->start_time = pad->priv->end_time = -1;
           if (!is_eos) {
-            GST_DEBUG_OBJECT (pad, "I just need more data");
-            if (GST_CLOCK_TIME_IS_VALID (pad->priv->max_last_buffer_repeat)) {
-              if (output_start_running_time - pad->priv->end_time >
-                  pad->priv->max_last_buffer_repeat) {
-                pad->priv->start_time = pad->priv->end_time = -1;
-                gst_buffer_replace (&pad->priv->buffer, NULL);
-                gst_caps_replace (&pad->priv->caps, NULL);
-              }
-            } else {
-              pad->priv->start_time = pad->priv->end_time = -1;
-            }
+            GST_DEBUG ("I just need more data");
             need_more_data = TRUE;
           } else {
             gst_buffer_replace (&pad->priv->buffer, NULL);
-            gst_caps_replace (&pad->priv->caps, NULL);
-            pad->priv->start_time = pad->priv->end_time = -1;
           }
         } else if (is_eos) {
           eos = FALSE;
         }
       } else if (is_eos) {
         gst_buffer_replace (&pad->priv->buffer, NULL);
-        gst_caps_replace (&pad->priv->caps, NULL);
-      } else if (pad->priv->start_time != -1) {
-        /* When the current buffer didn't have a duration, but
-         * max-last-buffer-repeat was set, we use start_time as
-         * the comparison point
-         */
-        if (pad->priv->start_time <= output_start_running_time) {
-          if (GST_CLOCK_TIME_IS_VALID (pad->priv->max_last_buffer_repeat)) {
-            if (output_start_running_time - pad->priv->start_time >
-                pad->priv->max_last_buffer_repeat) {
-              pad->priv->start_time = pad->priv->end_time = -1;
-              gst_buffer_replace (&pad->priv->buffer, NULL);
-              gst_caps_replace (&pad->priv->caps, NULL);
-            }
-          }
-        }
       }
     }
   }
@@ -1764,8 +1579,6 @@ gst_video_aggregator_fill_queues (GstVideoAggregator * vagg,
 
   if (need_more_data)
     return GST_AGGREGATOR_FLOW_NEED_DATA;
-  if (eos && !has_no_repeat_pads && repeat_pad_eos)
-    eos = FALSE;
   if (eos)
     return GST_FLOW_EOS;
 
@@ -1796,13 +1609,6 @@ prepare_frames (GstElement * agg, GstPad * pad, gpointer user_data)
   if (vpad->priv->buffer == NULL || !vaggpad_class->prepare_frame)
     return TRUE;
 
-  /* GAP event, nothing to do */
-  if (vpad->priv->buffer &&
-      gst_buffer_get_size (vpad->priv->buffer) == 0 &&
-      GST_BUFFER_FLAG_IS_SET (vpad->priv->buffer, GST_BUFFER_FLAG_GAP)) {
-    return TRUE;
-  }
-
   return vaggpad_class->prepare_frame (vpad, GST_VIDEO_AGGREGATOR_CAST (agg),
       vpad->priv->buffer, &vpad->priv->prepared_frame);
 }
@@ -1826,7 +1632,7 @@ clean_pad (GstElement * agg, GstPad * pad, gpointer user_data)
 static GstFlowReturn
 gst_video_aggregator_do_aggregate (GstVideoAggregator * vagg,
     GstClockTime output_start_time, GstClockTime output_end_time,
-    GstClockTime output_start_running_time, GstBuffer ** outbuf)
+    GstBuffer ** outbuf)
 {
   GstAggregator *agg = GST_AGGREGATOR (vagg);
   GstFlowReturn ret = GST_FLOW_OK;
@@ -1860,10 +1666,6 @@ gst_video_aggregator_do_aggregate (GstVideoAggregator * vagg,
   gst_element_foreach_sink_pad (GST_ELEMENT_CAST (vagg), sync_pad_values,
       &out_stream_time);
 
-  /* Let the application know that input buffers have been staged */
-  gst_aggregator_selected_samples (agg, GST_BUFFER_PTS (*outbuf),
-      GST_BUFFER_DTS (*outbuf), GST_BUFFER_DURATION (*outbuf), NULL);
-
   /* Convert all the frames the subclass has before aggregating */
   gst_element_foreach_sink_pad (GST_ELEMENT_CAST (vagg), prepare_frames, NULL);
 
@@ -2006,7 +1808,7 @@ gst_video_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
   } else {
     flow_ret =
         gst_video_aggregator_fill_queues (vagg, output_start_running_time,
-        output_end_running_time, timeout);
+        output_end_running_time);
   }
 
   if (flow_ret == GST_AGGREGATOR_FLOW_NEED_DATA && !timeout) {
@@ -2043,7 +1845,7 @@ gst_video_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
   jitter = gst_video_aggregator_do_qos (vagg, output_start_time);
   if (jitter <= 0) {
     flow_ret = gst_video_aggregator_do_aggregate (vagg, output_start_time,
-        output_end_time, output_start_running_time, &outbuf);
+        output_end_time, &outbuf);
     if (flow_ret != GST_FLOW_OK)
       goto done;
     vagg->priv->qos_processed++;
@@ -2402,8 +2204,6 @@ gst_video_aggregator_release_pad (GstElement * element, GstPad * pad)
     gst_video_aggregator_reset (vagg);
 
   gst_buffer_replace (&vaggpad->priv->buffer, NULL);
-  gst_caps_replace (&vaggpad->priv->caps, NULL);
-  gst_caps_replace (&vaggpad->priv->pending_caps, NULL);
 
   GST_ELEMENT_CLASS (gst_video_aggregator_parent_class)->release_pad
       (GST_ELEMENT (vagg), pad);
@@ -2574,7 +2374,10 @@ gst_video_aggregator_pad_sink_acceptcaps (GstPad * pad,
     GstVideoAggregator * vagg, GstCaps * caps)
 {
   gboolean ret;
+  GstCaps *modified_caps;
   GstCaps *accepted_caps;
+  GstCaps *template_caps;
+  gboolean had_current_caps = TRUE;
   gint i, n;
   GstStructure *s;
   GstAggregator *agg = GST_AGGREGATOR (vagg);
@@ -2583,8 +2386,12 @@ gst_video_aggregator_pad_sink_acceptcaps (GstPad * pad,
 
   accepted_caps = gst_pad_get_current_caps (GST_PAD (agg->srcpad));
 
-  if (accepted_caps == NULL)
-    accepted_caps = gst_pad_get_pad_template_caps (GST_PAD (agg->srcpad));
+  template_caps = gst_pad_get_pad_template_caps (GST_PAD (agg->srcpad));
+
+  if (accepted_caps == NULL) {
+    accepted_caps = template_caps;
+    had_current_caps = FALSE;
+  }
 
   accepted_caps = gst_caps_make_writable (accepted_caps);
 
@@ -2593,21 +2400,23 @@ gst_video_aggregator_pad_sink_acceptcaps (GstPad * pad,
   n = gst_caps_get_size (accepted_caps);
   for (i = 0; i < n; i++) {
     s = gst_caps_get_structure (accepted_caps, i);
-    gst_structure_set (s, "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT,
-        1, NULL);
+    gst_structure_set (s, "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+        "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+        "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
 
-    if (GST_IS_VIDEO_AGGREGATOR_CONVERT_PAD (pad)) {
-      gst_structure_set (s, "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-          "height", GST_TYPE_INT_RANGE, 1, G_MAXINT, NULL);
-      gst_structure_remove_fields (s, "colorimetry", "chroma-site", "format",
-          "pixel-aspect-ratio", NULL);
-    }
+    gst_structure_remove_fields (s, "colorimetry", "chroma-site", "format",
+        "pixel-aspect-ratio", NULL);
   }
 
+  modified_caps = gst_caps_intersect (accepted_caps, template_caps);
+
   ret = gst_caps_can_intersect (caps, accepted_caps);
   GST_DEBUG_OBJECT (pad, "%saccepted caps %" GST_PTR_FORMAT,
       (ret ? "" : "not "), caps);
   gst_caps_unref (accepted_caps);
+  gst_caps_unref (modified_caps);
+  if (had_current_caps)
+    gst_caps_unref (template_caps);
   return ret;
 }
 
@@ -2659,7 +2468,6 @@ gst_video_aggregator_finalize (GObject * o)
   GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (o);
 
   g_mutex_clear (&vagg->priv->lock);
-  g_ptr_array_unref (vagg->priv->supported_formats);
 
   G_OBJECT_CLASS (gst_video_aggregator_parent_class)->finalize (o);
 }
@@ -2739,7 +2547,6 @@ gst_video_aggregator_class_init (GstVideoAggregatorClass * klass)
       gst_video_aggregator_default_negotiated_src_caps;
   agg_class->decide_allocation = gst_video_aggregator_decide_allocation;
   agg_class->propose_allocation = gst_video_aggregator_propose_allocation;
-  agg_class->peek_next_sample = gst_video_aggregator_peek_next_sample;
 
   klass->find_best_format = gst_video_aggregator_find_best_format;
   klass->create_output_buffer = gst_video_aggregator_create_output_buffer;
@@ -2753,48 +2560,12 @@ static void
 gst_video_aggregator_init (GstVideoAggregator * vagg,
     GstVideoAggregatorClass * klass)
 {
-  GstCaps *src_template;
-  GstPadTemplate *pad_template;
-
   vagg->priv = gst_video_aggregator_get_instance_private (vagg);
+
   vagg->priv->current_caps = NULL;
 
   g_mutex_init (&vagg->priv->lock);
 
   /* initialize variables */
   gst_video_aggregator_reset (vagg);
-
-  /* Finding all supported formats */
-  vagg->priv->supported_formats = g_ptr_array_new ();
-  pad_template =
-      gst_element_class_get_pad_template (GST_ELEMENT_CLASS (klass), "src");
-  src_template = gst_pad_template_get_caps (pad_template);
-  for (gint i = 0; i < gst_caps_get_size (src_template); i++) {
-    const GValue *v =
-        gst_structure_get_value (gst_caps_get_structure (src_template, i),
-        "format");
-
-    if (G_VALUE_HOLDS_STRING (v)) {
-      GstVideoFormat f = gst_video_format_from_string (g_value_get_string (v));
-      GstVideoFormatInfo *format_info =
-          (GstVideoFormatInfo *) gst_video_format_get_info (f);
-      g_ptr_array_add (vagg->priv->supported_formats, format_info);
-      continue;
-    }
-
-    if (GST_VALUE_HOLDS_LIST (v)) {
-      gint j;
-
-      for (j = 0; j < gst_value_list_get_size (v); j++) {
-        const GValue *v1 = gst_value_list_get_value (v, j);
-        GstVideoFormat f =
-            gst_video_format_from_string (g_value_get_string (v1));
-        GstVideoFormatInfo *format_info =
-            (GstVideoFormatInfo *) gst_video_format_get_info (f);
-        g_ptr_array_add (vagg->priv->supported_formats, format_info);
-      }
-    }
-  }
-
-  gst_caps_unref (src_template);
 }
