@@ -26,15 +26,21 @@
 #include <gst/base/gstbytereader.h>
 #include "video-anc.h"
 
+#if !GLIB_CHECK_VERSION(2, 47, 4)
+#ifdef __GNUC__
+#define G_GNUC_CHECK_VERSION(major, minor) \
+    ((__GNUC__ > (major)) || \
+     ((__GNUC__ == (major)) && \
+      (__GNUC_MINOR__ >= (minor))))
+#else
+#define G_GNUC_CHECK_VERSION(major, minor) 0
+#endif
+#endif
+
 /**
  * SECTION:gstvideoanc
  * @title: GstVideo Ancillary
  * @short_description: Utilities for Ancillary data, VBI and Closed Caption
- * @private_symbols:
- * - GST_ANCILLARY_META_INFO
- * - GST_ANCILLARY_META_API_TYPE
- * - gst_ancillary_meta_get_info
- * - gst_ancillary_meta_api_get_type
  *
  * A collection of objects and methods to assist with handling Ancillary Data
  * present in Vertical Blanking Interval as well as Closed Caption.
@@ -120,7 +126,7 @@ get_ancillary_16 (GstVideoVBIParser * parser, GstVideoAncillary * anc)
       composite = TRUE;
     } else if (data[parser->offset] == 0x000 &&
         data[parser->offset + 1] == 0x3ff &&
-        data[parser->offset + 2] == 0x3ff) {
+        data[parser->offset + 1] == 0x3ff) {
       /* component */
       i += 3;
       composite = FALSE;
@@ -228,7 +234,7 @@ get_ancillary_8 (GstVideoVBIParser * parser, GstVideoAncillary * anc)
       composite = TRUE;
       i += 1;
     } else if (data[parser->offset] == 0x00 &&
-        data[parser->offset + 1] == 0xff && data[parser->offset + 2] == 0xff) {
+        data[parser->offset + 1] == 0xff && data[parser->offset + 1] == 0xff) {
       /* component */
       composite = FALSE;
       i += 3;
@@ -331,7 +337,7 @@ gst_video_vbi_parser_get_ancillary (GstVideoVBIParser * parser,
  *
  * Since: 1.16
  *
- * Returns: (nullable): The new #GstVideoVBIParser or %NULL if the @format and/or @pixel_width
+ * Returns: The new #GstVideoVBIParser or %NULL if the @format and/or @pixel_width
  * is not supported.
  */
 GstVideoVBIParser *
@@ -398,24 +404,13 @@ convert_line_from_uyvy (GstVideoVBIParser * parser, const guint8 * data)
 {
   guint i;
   guint8 *y = parser->work_data;
+  guint8 *uv = y + parser->info.width;
 
-  /* Data is stored differently in SD, making no distinction between Y and UV */
-  if (parser->info.width < 1280) {
-    for (i = 0; i < parser->info.width - 3; i += 4) {
-      *y++ = data[(i / 4) * 4 + 0];
-      *y++ = data[(i / 4) * 4 + 1];
-      *y++ = data[(i / 4) * 4 + 2];
-      *y++ = data[(i / 4) * 4 + 3];
-    }
-  } else {
-    guint8 *uv = y + parser->info.width;
-
-    for (i = 0; i < parser->info.width - 3; i += 4) {
-      *uv++ = data[(i / 4) * 4 + 0];
-      *y++ = data[(i / 4) * 4 + 1];
-      *uv++ = data[(i / 4) * 4 + 2];
-      *y++ = data[(i / 4) * 4 + 3];
-    }
+  for (i = 0; i < parser->info.width - 3; i += 4) {
+    *uv++ = data[(i / 4) * 4 + 0];
+    *y++ = data[(i / 4) * 4 + 1];
+    *uv++ = data[(i / 4) * 4 + 2];
+    *y++ = data[(i / 4) * 4 + 3];
   }
   GST_MEMDUMP ("Converted line", parser->work_data, 128);
 }
@@ -451,57 +446,30 @@ convert_line_from_v210 (GstVideoVBIParser * parser, const guint8 * data)
 {
   guint i;
   guint16 *y = (guint16 *) parser->work_data;
+  guint16 *uv = y + parser->info.width;
   guint32 a, b, c, d;
 
-  /* Data is stored differently in SD, making no distinction between Y and UV */
-  if (parser->info.width < 1280) {
-    /* Convert the line */
-    for (i = 0; i < parser->info.width - 5; i += 6) {
-      a = GST_READ_UINT32_LE (data + (i / 6) * 16 + 0);
-      b = GST_READ_UINT32_LE (data + (i / 6) * 16 + 4);
-      c = GST_READ_UINT32_LE (data + (i / 6) * 16 + 8);
-      d = GST_READ_UINT32_LE (data + (i / 6) * 16 + 12);
+  /* Convert the line */
+  for (i = 0; i < parser->info.width - 5; i += 6) {
+    a = GST_READ_UINT32_LE (data + (i / 6) * 16 + 0);
+    b = GST_READ_UINT32_LE (data + (i / 6) * 16 + 4);
+    c = GST_READ_UINT32_LE (data + (i / 6) * 16 + 8);
+    d = GST_READ_UINT32_LE (data + (i / 6) * 16 + 12);
 
-      *y++ = (a >> 0) & 0x3ff;
-      *y++ = (a >> 10) & 0x3ff;
-      *y++ = (a >> 20) & 0x3ff;
-      *y++ = (b >> 0) & 0x3ff;
+    *uv++ = (a >> 0) & 0x3ff;
+    *y++ = (a >> 10) & 0x3ff;
+    *uv++ = (a >> 20) & 0x3ff;
+    *y++ = (b >> 0) & 0x3ff;
 
-      *y++ = (b >> 10) & 0x3ff;
-      *y++ = (b >> 20) & 0x3ff;
-      *y++ = (c >> 0) & 0x3ff;
-      *y++ = (c >> 10) & 0x3ff;
+    *uv++ = (b >> 10) & 0x3ff;
+    *y++ = (b >> 20) & 0x3ff;
+    *uv++ = (c >> 0) & 0x3ff;
+    *y++ = (c >> 10) & 0x3ff;
 
-      *y++ = (c >> 20) & 0x3ff;
-      *y++ = (d >> 0) & 0x3ff;
-      *y++ = (d >> 10) & 0x3ff;
-      *y++ = (d >> 20) & 0x3ff;
-    }
-  } else {
-    guint16 *uv = y + parser->info.width;
-
-    /* Convert the line */
-    for (i = 0; i < parser->info.width - 5; i += 6) {
-      a = GST_READ_UINT32_LE (data + (i / 6) * 16 + 0);
-      b = GST_READ_UINT32_LE (data + (i / 6) * 16 + 4);
-      c = GST_READ_UINT32_LE (data + (i / 6) * 16 + 8);
-      d = GST_READ_UINT32_LE (data + (i / 6) * 16 + 12);
-
-      *uv++ = (a >> 0) & 0x3ff;
-      *y++ = (a >> 10) & 0x3ff;
-      *uv++ = (a >> 20) & 0x3ff;
-      *y++ = (b >> 0) & 0x3ff;
-
-      *uv++ = (b >> 10) & 0x3ff;
-      *y++ = (b >> 20) & 0x3ff;
-      *uv++ = (c >> 0) & 0x3ff;
-      *y++ = (c >> 10) & 0x3ff;
-
-      *uv++ = (c >> 20) & 0x3ff;
-      *y++ = (d >> 0) & 0x3ff;
-      *uv++ = (d >> 10) & 0x3ff;
-      *y++ = (d >> 20) & 0x3ff;
-    }
+    *uv++ = (c >> 20) & 0x3ff;
+    *y++ = (d >> 0) & 0x3ff;
+    *uv++ = (d >> 10) & 0x3ff;
+    *y++ = (d >> 20) & 0x3ff;
   }
 
   if (0) {
@@ -612,7 +580,7 @@ gst_video_vbi_encoder_free (GstVideoVBIEncoder * encoder)
  *
  * Since: 1.16
  *
- * Returns: (nullable): The new #GstVideoVBIEncoder or %NULL if the @format and/or @pixel_width
+ * Returns: The new #GstVideoVBIEncoder or %NULL if the @format and/or @pixel_width
  * is not supported.
  */
 GstVideoVBIEncoder *
@@ -799,71 +767,39 @@ convert_line_to_v210 (GstVideoVBIEncoder * encoder, guint8 * data)
 {
   guint i;
   const guint16 *y = (const guint16 *) encoder->work_data;
+  const guint16 *uv = y + encoder->info.width;
   guint32 a, b, c, d;
 
-  /* Data is stored differently in SD, making no distinction between Y and UV */
-  if (encoder->info.width < 1280) {
-    /* Convert the line */
-    for (i = 0; i < encoder->info.width - 5; i += 6) {
-      a = ((y[0] & 0x3ff) << 0)
-          | ((y[1] & 0x3ff) << 10)
-          | ((y[2] & 0x3ff) << 20);
-      y += 3;
+  /* Convert the line */
+  for (i = 0; i < encoder->info.width - 5; i += 6) {
+    a = ((uv[0] & 0x3ff) << 0)
+        | ((y[0] & 0x3ff) << 10)
+        | ((uv[1] & 0x3ff) << 20);
+    uv += 2;
+    y++;
 
-      b = ((y[0] & 0x3ff) << 0)
-          | ((y[1] & 0x3ff) << 10)
-          | ((y[2] & 0x3ff) << 20);
-      y += 3;
+    b = ((y[0] & 0x3ff) << 0)
+        | ((uv[0] & 0x3ff) << 10)
+        | ((y[1] & 0x3ff) << 20);
+    y += 2;
+    uv++;
 
-      c = ((y[0] & 0x3ff) << 0)
-          | ((y[1] & 0x3ff) << 10)
-          | ((y[2] & 0x3ff) << 20);
-      y += 3;
+    c = ((uv[0] & 0x3ff) << 0)
+        | ((y[0] & 0x3ff) << 10)
+        | ((uv[1] & 0x3ff) << 20);
+    uv += 2;
+    y++;
 
-      d = ((y[0] & 0x3ff) << 0)
-          | ((y[1] & 0x3ff) << 10)
-          | ((y[2] & 0x3ff) << 20);
-      y += 3;
+    d = ((y[0] & 0x3ff) << 0)
+        | ((uv[0] & 0x3ff) << 10)
+        | ((y[1] & 0x3ff) << 20);
+    y += 2;
+    uv++;
 
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 0, a);
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 4, b);
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 8, c);
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 12, d);
-    }
-  } else {
-    const guint16 *uv = y + encoder->info.width;
-
-    /* Convert the line */
-    for (i = 0; i < encoder->info.width - 5; i += 6) {
-      a = ((uv[0] & 0x3ff) << 0)
-          | ((y[0] & 0x3ff) << 10)
-          | ((uv[1] & 0x3ff) << 20);
-      uv += 2;
-      y++;
-
-      b = ((y[0] & 0x3ff) << 0)
-          | ((uv[0] & 0x3ff) << 10)
-          | ((y[1] & 0x3ff) << 20);
-      y += 2;
-      uv++;
-
-      c = ((uv[0] & 0x3ff) << 0)
-          | ((y[0] & 0x3ff) << 10)
-          | ((uv[1] & 0x3ff) << 20);
-      uv += 2;
-      y++;
-
-      d = ((y[0] & 0x3ff) << 0)
-          | ((uv[0] & 0x3ff) << 10)
-          | ((y[1] & 0x3ff) << 20);
-      y += 2;
-      uv++;
-
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 0, a);
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 4, b);
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 8, c);
-      GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 12, d);
-    }
+    GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 0, a);
+    GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 4, b);
+    GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 8, c);
+    GST_WRITE_UINT32_LE (data + (i / 6) * 16 + 12, d);
   }
 }
 
@@ -872,35 +808,16 @@ convert_line_to_uyvy (GstVideoVBIEncoder * encoder, guint8 * data)
 {
   guint i;
   const guint8 *y = encoder->work_data;
+  const guint8 *uv = y + encoder->info.width;
 
-  /* Data is stored differently in SD, making no distinction between Y and UV */
-  if (encoder->info.width < 1280) {
-    for (i = 0; i < encoder->info.width - 3; i += 4) {
-      data[(i / 4) * 4 + 0] = *y++;
-      data[(i / 4) * 4 + 1] = *y++;
-      data[(i / 4) * 4 + 2] = *y++;
-      data[(i / 4) * 4 + 3] = *y++;
-    }
-  } else {
-    const guint8 *uv = y + encoder->info.width;
-
-    for (i = 0; i < encoder->info.width - 3; i += 4) {
-      data[(i / 4) * 4 + 0] = *uv++;
-      data[(i / 4) * 4 + 1] = *y++;
-      data[(i / 4) * 4 + 2] = *uv++;
-      data[(i / 4) * 4 + 3] = *y++;
-    }
+  for (i = 0; i < encoder->info.width - 3; i += 4) {
+    data[(i / 4) * 4 + 0] = *uv++;
+    data[(i / 4) * 4 + 1] = *y++;
+    data[(i / 4) * 4 + 2] = *uv++;
+    data[(i / 4) * 4 + 3] = *y++;
   }
 }
 
-/**
- * gst_video_vbi_encoder_write_line:
- * @encoder: a #GstVideoVBIEncoder
- * @data: (array): The line to write to
- *
- * @data needs to have the correct size and alignment for the configured video
- * format of @encoder.
- */
 void
 gst_video_vbi_encoder_write_line (GstVideoVBIEncoder * encoder, guint8 * data)
 {
@@ -934,7 +851,7 @@ gst_video_vbi_encoder_write_line (GstVideoVBIEncoder * encoder, guint8 * data)
 GType
 gst_video_caption_meta_api_get_type (void)
 {
-  static GType type = 0;
+  static volatile GType type;
 
   if (g_once_init_enter (&type)) {
     static const gchar *tags[] = { NULL };
@@ -1044,7 +961,7 @@ gst_buffer_add_video_caption_meta (GstBuffer * buffer,
   g_return_val_if_fail (meta != NULL, NULL);
 
   meta->caption_type = caption_type;
-  meta->data = g_memdup2 (data, size);
+  meta->data = g_memdup (data, size);
   meta->size = size;
 
   return meta;
@@ -1131,365 +1048,3 @@ gst_video_caption_type_to_caps (GstVideoCaptionType type)
 
   return caption_caps;
 }
-
-/* Ancillary Meta Implementation */
-
-GType
-gst_ancillary_meta_api_get_type (void)
-{
-  static GType type = 0;
-
-  if (g_once_init_enter (&type)) {
-    static const gchar *tags[] = { NULL };
-    GType _type = gst_meta_api_type_register ("GstAncillaryMetaAPI", tags);
-    GST_INFO ("registering");
-    g_once_init_leave (&type, _type);
-  }
-  return type;
-}
-
-static gboolean
-gst_ancillary_meta_init (GstMeta * meta, gpointer params, GstBuffer * buffer)
-{
-  GstAncillaryMeta *ameta = (GstAncillaryMeta *) meta;
-
-  /* Set sensible default values */
-  ameta->field = GST_ANCILLARY_META_FIELD_PROGRESSIVE;
-  ameta->c_not_y_channel = 0;
-  ameta->line = 0x7ff;
-  ameta->offset = 0xfff;
-
-  ameta->DID = ameta->SDID_block_number = ameta->data_count = 0;
-  ameta->data = NULL;
-  ameta->checksum = 0;
-
-  return TRUE;
-}
-
-static void
-gst_ancillary_meta_free (GstMeta * meta, GstBuffer * buffer)
-{
-  GstAncillaryMeta *ameta = (GstAncillaryMeta *) meta;
-
-  g_free (ameta->data);
-}
-
-static gboolean
-gst_ancillary_meta_transform (GstBuffer * dest, GstMeta * meta,
-    GstBuffer * buffer, GQuark type, gpointer data)
-{
-  GstAncillaryMeta *dmeta, *smeta;
-
-  /* We always copy over the ancillary meta */
-  smeta = (GstAncillaryMeta *) meta;
-
-  dmeta =
-      (GstAncillaryMeta *) gst_buffer_add_meta (dest, GST_ANCILLARY_META_INFO,
-      NULL);
-
-  dmeta->field = smeta->field;
-  dmeta->c_not_y_channel = smeta->c_not_y_channel;
-  dmeta->line = smeta->line;
-  dmeta->offset = smeta->offset;
-  dmeta->DID = smeta->DID;
-  dmeta->SDID_block_number = smeta->SDID_block_number;
-  dmeta->data_count = smeta->data_count;
-  dmeta->data = g_memdup2 (smeta->data, (smeta->data_count & 0xff) * 2);
-  dmeta->checksum = smeta->checksum;
-
-  return TRUE;
-}
-
-const GstMetaInfo *
-gst_ancillary_meta_get_info (void)
-{
-  static const GstMetaInfo *meta_info = NULL;
-
-  if (g_once_init_enter ((GstMetaInfo **) & meta_info)) {
-    const GstMetaInfo *mi = gst_meta_register (GST_ANCILLARY_META_API_TYPE,
-        "GstAncillaryMeta",
-        sizeof (GstAncillaryMeta),
-        gst_ancillary_meta_init,
-        gst_ancillary_meta_free,
-        gst_ancillary_meta_transform);
-    g_once_init_leave ((GstMetaInfo **) & meta_info, (GstMetaInfo *) mi);
-  }
-  return meta_info;
-
-}
-
-/**
- * gst_buffer_add_ancillary_meta:
- * @buffer: A #GstBuffer
- *
- * Adds a new #GstAncillaryMeta to the @buffer. The caller is responsible for setting the appropriate
- * fields.
- *
- * Since: 1.24
- *
- * Returns: (transfer none): A new #GstAncillaryMeta, or %NULL if an error happened.
- */
-
-GstAncillaryMeta *
-gst_buffer_add_ancillary_meta (GstBuffer * buffer)
-{
-  GstAncillaryMeta *meta;
-
-  meta =
-      (GstAncillaryMeta *) gst_buffer_add_meta (buffer, GST_ANCILLARY_META_INFO,
-      NULL);
-  g_assert (meta != NULL);
-
-  return meta;
-}
-
-/* Active Format Description (AFD) Meta implementation */
-
-GType
-gst_video_afd_meta_api_get_type (void)
-{
-  static GType type = 0;
-
-  if (g_once_init_enter (&type)) {
-    static const gchar *tags[] = {
-      GST_META_TAG_VIDEO_SIZE_STR,
-      GST_META_TAG_VIDEO_ORIENTATION_STR,
-      GST_META_TAG_VIDEO_STR,
-      NULL
-    };
-    GType _type = gst_meta_api_type_register ("GstVideoAFDMetaAPI", tags);
-    g_once_init_leave (&type, _type);
-  }
-  return type;
-}
-
-static gboolean
-gst_video_afd_meta_init (GstMeta * meta, gpointer params, GstBuffer * buffer)
-{
-  GstVideoAFDMeta *emeta = (GstVideoAFDMeta *) meta;
-
-  emeta->field = 0;
-  emeta->spec = GST_VIDEO_AFD_SPEC_ATSC_A53;
-  emeta->afd = GST_VIDEO_AFD_UNAVAILABLE;
-
-  return TRUE;
-}
-
-static gboolean
-gst_video_afd_meta_transform (GstBuffer * dest, GstMeta * meta,
-    GstBuffer * buffer, GQuark type, gpointer data)
-{
-  GstVideoAFDMeta *smeta = (GstVideoAFDMeta *) meta;
-
-  if (GST_META_TRANSFORM_IS_COPY (type)) {
-    GST_DEBUG ("copy AFD metadata");
-    gst_buffer_add_video_afd_meta (dest, smeta->field, smeta->spec, smeta->afd);
-    return TRUE;
-  } else if (GST_VIDEO_META_TRANSFORM_IS_SCALE (type)) {
-    GstVideoMetaTransform *trans = data;
-    gdouble diff;
-    gint ow, oh, nw, nh;
-    gint opn, opd, npn, npd;
-
-    ow = GST_VIDEO_INFO_WIDTH (trans->in_info);
-    nw = GST_VIDEO_INFO_WIDTH (trans->out_info);
-    oh = GST_VIDEO_INFO_HEIGHT (trans->in_info);
-    nh = GST_VIDEO_INFO_HEIGHT (trans->out_info);
-    opn = GST_VIDEO_INFO_PAR_N (trans->in_info);
-    opd = GST_VIDEO_INFO_PAR_D (trans->in_info);
-    npn = GST_VIDEO_INFO_PAR_N (trans->out_info);
-    npd = GST_VIDEO_INFO_PAR_D (trans->out_info);
-
-    /* if the aspect ratio stays the same we can copy the meta, otherwise
-     * we can't know if the aspect ratio was changed or black borders were
-     * introduced. Both would invalidate the AFD meta */
-
-    diff =
-        ABS (((gdouble) ow / (gdouble) oh) * ((gdouble) opn / (gdouble) opd) -
-        ((gdouble) nw / (gdouble) nh) * ((gdouble) npn / (gdouble) npd));
-    if (diff < 0.0001) {
-      GST_DEBUG ("copying AFD metadata, aspect ratio did not change");
-      gst_buffer_add_video_afd_meta (dest, smeta->field, smeta->spec,
-          smeta->afd);
-      return TRUE;
-    } else {
-      return FALSE;
-    }
-  } else {
-    /* return FALSE, if transform type is not supported */
-    return FALSE;
-  }
-
-}
-
-const GstMetaInfo *
-gst_video_afd_meta_get_info (void)
-{
-  static const GstMetaInfo *meta_info = NULL;
-
-  if (g_once_init_enter ((GstMetaInfo **) & meta_info)) {
-    const GstMetaInfo *mi = gst_meta_register (GST_VIDEO_AFD_META_API_TYPE,
-        "GstVideoAFDMeta",
-        sizeof (GstVideoAFDMeta),
-        gst_video_afd_meta_init,
-        NULL,
-        gst_video_afd_meta_transform);
-    g_once_init_leave ((GstMetaInfo **) & meta_info, (GstMetaInfo *) mi);
-  }
-  return meta_info;
-}
-
-/**
- * gst_buffer_add_video_afd_meta:
- * @buffer: a #GstBuffer
- * @field: 0 for progressive or field 1 and 1 for field 2
- * @spec: #GstVideoAFDSpec that applies to AFD value
- * @afd: #GstVideoAFDValue AFD enumeration
- *
- * Attaches #GstVideoAFDMeta metadata to @buffer with the given
- * parameters.
- *
- * Returns: (transfer none): the #GstVideoAFDMeta on @buffer.
- *
- * Since: 1.18
- */
-GstVideoAFDMeta *
-gst_buffer_add_video_afd_meta (GstBuffer * buffer, guint8 field,
-    GstVideoAFDSpec spec, GstVideoAFDValue afd)
-{
-  GstVideoAFDMeta *meta;
-  gint8 afd_data GST_UNUSED_CHECKS = (gint8) afd;
-  g_return_val_if_fail (GST_IS_BUFFER (buffer), NULL);
-  g_return_val_if_fail (field <= 1, NULL);
-  g_return_val_if_fail ((guint8) spec <= 2, NULL);
-  /* AFD is stored in a nybble */
-  g_return_val_if_fail (afd_data <= 0xF, NULL);
-  /* reserved values for all specifications */
-  g_return_val_if_fail (afd_data != 1 && (afd_data < 5 || afd_data > 7)
-      && afd_data != 12, NULL);
-  /* reserved for DVB/ETSI */
-  g_return_val_if_fail ((spec != GST_VIDEO_AFD_SPEC_DVB_ETSI)
-      || (afd_data != 0), NULL);
-
-  meta = (GstVideoAFDMeta *) gst_buffer_add_meta (buffer,
-      GST_VIDEO_AFD_META_INFO, NULL);
-  g_assert (meta != NULL);
-
-  meta->field = field;
-  meta->spec = spec;
-  meta->afd = afd;
-
-  return meta;
-}
-
-/* Bar Meta implementation */
-
-GType
-gst_video_bar_meta_api_get_type (void)
-{
-  static GType type = 0;
-
-  if (g_once_init_enter (&type)) {
-    static const gchar *tags[] = {
-      GST_META_TAG_VIDEO_SIZE_STR,
-      GST_META_TAG_VIDEO_ORIENTATION_STR,
-      GST_META_TAG_VIDEO_STR,
-      NULL
-    };
-    GType _type = gst_meta_api_type_register ("GstVideoBarMetaAPI", tags);
-    g_once_init_leave (&type, _type);
-  }
-  return type;
-}
-
-static gboolean
-gst_video_bar_meta_init (GstMeta * meta, gpointer params, GstBuffer * buffer)
-{
-  GstVideoBarMeta *emeta = (GstVideoBarMeta *) meta;
-
-  emeta->field = 0;
-  emeta->is_letterbox = FALSE;
-  emeta->bar_data1 = 0;
-  emeta->bar_data2 = 0;
-
-  return TRUE;
-}
-
-static gboolean
-gst_video_bar_meta_transform (GstBuffer * dest, GstMeta * meta,
-    GstBuffer * buffer, GQuark type, gpointer data)
-{
-  GstVideoBarMeta *smeta = (GstVideoBarMeta *) meta;
-
-  if (GST_META_TRANSFORM_IS_COPY (type)) {
-    GST_DEBUG ("copy Bar metadata");
-    gst_buffer_add_video_bar_meta (dest, smeta->field, smeta->is_letterbox,
-        smeta->bar_data1, smeta->bar_data2);
-    return TRUE;
-  } else {
-    /* return FALSE, if transform type is not supported */
-    return FALSE;
-  }
-}
-
-const GstMetaInfo *
-gst_video_bar_meta_get_info (void)
-{
-  static const GstMetaInfo *meta_info = NULL;
-
-  if (g_once_init_enter ((GstMetaInfo **) & meta_info)) {
-    const GstMetaInfo *mi = gst_meta_register (GST_VIDEO_BAR_META_API_TYPE,
-        "GstVideoBarMeta",
-        sizeof (GstVideoBarMeta),
-        gst_video_bar_meta_init,
-        NULL,
-        gst_video_bar_meta_transform);
-    g_once_init_leave ((GstMetaInfo **) & meta_info, (GstMetaInfo *) mi);
-  }
-  return meta_info;
-}
-
-/**
- * gst_buffer_add_video_bar_meta:
- * @buffer: a #GstBuffer
- * @field: 0 for progressive or field 1 and 1 for field 2
- * @is_letterbox: if true then bar data specifies letterbox, otherwise pillarbox
- * @bar_data1: If @is_letterbox is true, then the value specifies the
- *      last line of a horizontal letterbox bar area at top of reconstructed frame.
- *      Otherwise, it specifies the last horizontal luminance sample of a vertical pillarbox
- *      bar area at the left side of the reconstructed frame
- * @bar_data2: If @is_letterbox is true, then the value specifies the
- *      first line of a horizontal letterbox bar area at bottom of reconstructed frame.
- *      Otherwise, it specifies the first horizontal
- *      luminance sample of a vertical pillarbox bar area at the right side of the reconstructed frame.
- *
- * Attaches #GstVideoBarMeta metadata to @buffer with the given
- * parameters.
- *
- * Returns: (transfer none): the #GstVideoBarMeta on @buffer.
- *
- * See Table 6.11 Bar Data Syntax
- *
- * https://www.atsc.org/wp-content/uploads/2015/03/a_53-Part-4-2009.pdf
- *
- * Since: 1.18
- */
-GstVideoBarMeta *
-gst_buffer_add_video_bar_meta (GstBuffer * buffer, guint8 field,
-    gboolean is_letterbox, guint bar_data1, guint bar_data2)
-{
-  GstVideoBarMeta *meta;
-
-  g_return_val_if_fail (GST_IS_BUFFER (buffer), NULL);
-  g_return_val_if_fail (field <= 1, NULL);
-
-  meta = (GstVideoBarMeta *) gst_buffer_add_meta (buffer,
-      GST_VIDEO_BAR_META_INFO, NULL);
-  g_assert (meta != NULL);
-
-  meta->field = field;
-  meta->is_letterbox = is_letterbox;
-  meta->bar_data1 = bar_data1;
-  meta->bar_data2 = bar_data2;
-  return meta;
-}
