@@ -75,6 +75,7 @@ gst_video_frame_map_id (GstVideoFrame * frame, GstVideoInfo * info,
 
   g_return_val_if_fail (frame != NULL, FALSE);
   g_return_val_if_fail (info != NULL, FALSE);
+  g_return_val_if_fail (info->finfo != NULL, FALSE);
   g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
 
   if (id == -1)
@@ -199,7 +200,7 @@ invalid_size:
  *   GstVideoFrame vframe;
  *   ...
  *   // set RGB pixels to black one at a time
- *   if (gst_video_frame_map (&amp;vframe, video_info, video_buffer)) {
+ *   if (gst_video_frame_map (&amp;vframe, video_info, video_buffer, GST_MAP_WRITE)) {
  *     guint8 *pixels = GST_VIDEO_FRAME_PLANE_DATA (vframe, 0);
  *     guint stride = GST_VIDEO_FRAME_PLANE_STRIDE (vframe, 0);
  *     guint pixel_stride = GST_VIDEO_FRAME_COMP_PSTRIDE (vframe, 0);
@@ -211,6 +212,8 @@ invalid_size:
  *         memset (pixel, 0, pixel_stride);
  *       }
  *     }
+ *
+ *     gst_video_frame_unmap (&amp;vframe);
  *   }
  *   ...
  * ]|
