@@ -70,27 +70,6 @@ static gchar rgba_data[] =
   RED, GREEN, BLUE, RED, GREEN, BLUE, RED, GREEN, BLUE, RED
 };
 
-#ifndef GST_CAPS_FEATURE_MEMORY_DMABUF
-#define GST_CAPS_FEATURE_MEMORY_DMABUF "memory:DMABuf"
-#endif
-
-static GstVideoFormat test_passthrough_formats[] = {
-  GST_VIDEO_FORMAT_DMA_DRM,
-};
-
-typedef struct
-{
-  const gchar *features[3];     /* NULL-terminated array of feature strings */
-} PassthroughFeatureSet;
-
-static PassthroughFeatureSet test_passthrough_feature_sets[] = {
-  /* Single feature: DMABuf only */
-  {{GST_CAPS_FEATURE_MEMORY_DMABUF, NULL}},
-  /* Multiple features: DMABuf + overlay composition */
-  {{GST_CAPS_FEATURE_MEMORY_DMABUF,
-          GST_CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION, NULL}},
-};
-
 static void
 setup (void)
 {
@@ -292,7 +271,7 @@ GST_START_TEST (test_upload_data)
   in_caps = gst_caps_from_string ("video/x-raw,format=RGBA,"
       "width=10,height=10");
   out_caps = gst_caps_from_string ("video/x-raw(memory:GLMemory),"
-      "format=RGBA,width=10,height=10,texture-target=2D");
+      "format=RGBA,width=10,height=10");
 
   gst_gl_upload_set_caps (upload, in_caps, out_caps);
 
@@ -302,8 +281,6 @@ GST_START_TEST (test_upload_data)
   res = gst_gl_upload_perform_with_buffer (upload, inbuf, &outbuf);
   fail_unless (res == GST_GL_UPLOAD_DONE, "Failed to upload buffer");
   fail_unless (GST_IS_BUFFER (outbuf));
-  fail_unless (gst_buffer_get_video_meta (outbuf));
-  fail_unless (gst_buffer_get_gl_sync_meta (outbuf));
 
   res = gst_buffer_map (outbuf, &map_info, GST_MAP_READ | GST_MAP_GL);
   fail_if (res == FALSE, "Failed to map gl memory");
@@ -342,6 +319,7 @@ GST_START_TEST (test_upload_gl_memory)
   GstStructure *out_s;
   GstVideoInfo in_info;
   GstMapInfo map_info;
+  gint i = 0;
   gint res;
 
   base_mem_alloc =
@@ -372,14 +350,9 @@ GST_START_TEST (test_upload_gl_memory)
   /* at this point glupload hasn't received any buffers so can output anything */
   out_caps = gst_gl_upload_transform_caps (upload, context,
       GST_PAD_SINK, in_caps, NULL);
-  for (gint i = 0; i < gst_caps_get_size (out_caps); i++) {
-    out_s = gst_caps_get_structure (out_caps, i);
-    res =
-        gst_structure_has_field_typed (out_s, "texture-target", GST_TYPE_LIST);
-    if (res)
-      break;
-  }
-  fail_unless (res);
+  out_s = gst_caps_get_structure (out_caps, 0);
+  fail_unless (gst_structure_has_field_typed (out_s, "texture-target",
+          GST_TYPE_LIST));
   gst_caps_unref (out_caps);
 
   /* set some output caps without setting texture-target: this should trigger RECONFIGURE */
@@ -403,7 +376,7 @@ GST_START_TEST (test_upload_gl_memory)
       "2D");
   gst_caps_unref (out_caps);
 
-  /* try setting the wrong type first */
+  /* try setting the wrong type first tho */
   out_caps = gst_caps_from_string ("video/x-raw(memory:GLMemory),"
       "format=RGBA,width=10,height=10,texture-target=RECTANGLE");
   gst_gl_upload_set_caps (upload, in_caps, out_caps);
@@ -427,9 +400,11 @@ GST_START_TEST (test_upload_gl_memory)
   gst_gl_window_draw (window);
   gst_gl_window_send_message (window, GST_GL_WINDOW_CB (init), context);
 
-  for (gint i = 0; i < 2; i++)
+  while (i < 2) {
     gst_gl_window_send_message (window, GST_GL_WINDOW_CB (draw_render),
         context);
+    i++;
+  }
   gst_gl_window_send_message (window, GST_GL_WINDOW_CB (deinit), context);
 
   gst_caps_unref (in_caps);
@@ -440,75 +415,6 @@ GST_START_TEST (test_upload_gl_memory)
 
 GST_END_TEST;
 
-GST_START_TEST (test_passthrough)
-{
-  guint formats_size = G_N_ELEMENTS (test_passthrough_formats);
-  guint features_size = G_N_ELEMENTS (test_passthrough_feature_sets);
-  gint i, j, k, l, m;
-
-  for (i = 0; i < formats_size; i++) {
-    GstVideoFormat in_format = test_passthrough_formats[i];
-
-    for (j = 0; j < formats_size; j++) {
-      GstVideoFormat out_format = test_passthrough_formats[j];
-
-      for (k = 0; k < features_size; k++) {
-        PassthroughFeatureSet *in_feature_set =
-            &test_passthrough_feature_sets[k];
-        GstCaps *in_caps;
-        GstCapsFeatures *in_features;
-
-        in_caps = gst_caps_new_simple ("video/x-raw", "format", G_TYPE_STRING,
-            gst_video_format_to_string (in_format), NULL);
-
-        in_features = gst_caps_features_new_empty ();
-        for (m = 0; in_feature_set->features[m] != NULL; m++) {
-          gst_caps_features_add (in_features, in_feature_set->features[m]);
-        }
-        gst_caps_set_features_simple (in_caps, in_features);
-
-        for (l = 0; l < features_size; l++) {
-          PassthroughFeatureSet *out_feature_set =
-              &test_passthrough_feature_sets[l];
-          GstCaps *out_caps;
-          GstCapsFeatures *out_features;
-
-          out_caps = gst_caps_new_simple ("video/x-raw", "format",
-              G_TYPE_STRING, gst_video_format_to_string (out_format), NULL);
-
-          out_features = gst_caps_features_new_empty ();
-          for (m = 0; out_feature_set->features[m] != NULL; m++) {
-            gst_caps_features_add (out_features, out_feature_set->features[m]);
-          }
-          gst_caps_set_features_simple (out_caps, out_features);
-
-          if (gst_caps_is_equal (in_caps, out_caps)) {
-            GstCaps *tmp_caps, *tmp_caps2, *tmp_caps3;
-
-            tmp_caps = gst_gl_upload_transform_caps (upload, context,
-                GST_PAD_SINK, in_caps, NULL);
-            tmp_caps2 = gst_gl_upload_transform_caps (upload, context,
-                GST_PAD_SRC, out_caps, NULL);
-
-            tmp_caps3 = gst_caps_intersect (tmp_caps, tmp_caps2);
-
-            fail_unless (!gst_caps_is_empty (tmp_caps3));
-
-            gst_caps_unref (tmp_caps);
-            gst_caps_unref (tmp_caps2);
-            gst_caps_unref (tmp_caps3);
-          }
-
-          gst_caps_unref (out_caps);
-        }
-
-        gst_caps_unref (in_caps);
-      }
-    }
-  }
-}
-
-GST_END_TEST;
 
 static Suite *
 gst_gl_upload_suite (void)
@@ -520,7 +426,6 @@ gst_gl_upload_suite (void)
   tcase_add_checked_fixture (tc_chain, setup, teardown);
   tcase_add_test (tc_chain, test_upload_data);
   tcase_add_test (tc_chain, test_upload_gl_memory);
-  tcase_add_test (tc_chain, test_passthrough);
 
   return s;
 }
